/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.client.cropfield.type;

import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropFieldManager;
import nl.cloudfarming.client.farm.model.FieldPlanManager;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

public class CropFieldDataObject extends MultiDataObject {
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.cropfield.cropfielddataobject");
    private InstanceContent ic;
    private Lookup intern;
    private CropFieldManager  cropFieldManager;
    
    public CropFieldDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(CropField.MIMETYPE, true);
        
        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);

        cropFieldManager = Lookup.getDefault().lookup(CropFieldManager.class);
        assert cropFieldManager != null : "No CropFieldManager class found in the default lookup, unable to load the DataObject";
    }
        
    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    protected Node createNodeDelegate() {
        CropField cropField = null;
        try {
            cropField = cropFieldManager.loadCropField(getPrimaryFile());
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
        assert cropField != null;
        LOGGER.log(Level.FINE, "CropFieldDataObject loaded {0}", System.identityHashCode(cropField));
        ic.add(cropField);
        LOGGER.log(Level.FINE, "Adding new SimpleLayer in FieldDataObject lookup");
        ic.add(new SimpleLayer(cropField.getName()));
        
        // cropField.addPropertyChangeListener(this);
        LOGGER.log(Level.FINE, "Creating new node delegate for CropFieldDataObject");           
        
        Node delegate = new CropFieldNode(super.createNodeDelegate());
        return delegate;
    }
     
    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }

    private static final class CropFieldNode extends FilterNode{

        public CropFieldNode(Node original) {
            super(original);
            assert original.getLookup().lookup(CropField.class)!= null:"Should be a CropField in the lookup of a CropFieldNode";
            //disableDelegation(DELEGATE_SET_DISPLAY_NAME);
            disableDelegation(DELEGATE_SET_NAME);
            setName(original.getLookup().lookup(CropField.class).getName());
            setDisplayName(getName());
        }

//        @Override
//        public Transferable drag() throws IOException {
//            return clipboardCopy();
//        }
//
//        @Override
//        public Transferable clipboardCopy() throws IOException {
//            return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
//        }

        @Override
        public HelpCtx getHelpCtx() {
            return new HelpCtx("nl.cloudfarming.client.cropfield.type.about");
        }
        
        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }
}
